#!/usr/bin/env python
#
# MOMAP toolkit, http://momap.net.cn
# Copyright (2018) Zhigang Shuai Group, Tsinghua University.
#
import os
import sys
import time
import glob
import shutil
import re
from getopt import getopt
from datetime import timedelta

start_time = time.time()

WORKDIR = os.getcwd()

# sys.stderr.write("Running direcotory: %s" % WORKDIR)

MOMAP_OS = ""
if sys.platform == "linux" or sys.platform == "linux2":
    MOMAP_OS = "LINUX"
elif sys.platform == "darwin":
    MOMAP_OS = "MACOS"
elif sys.platform == "win32":
    MOMAP_OS = "WIN"
elif sys.platform == "win64":
    MOMAP_OS = "WIN"

# -----------------------------------------------------------------------
# Initialization
#
inputfile = "momap.inp"
machinefile = ""
nodefile = ""
nprocs = 0

argv = sys.argv[1:]
try:
    opts, args = getopt(argv, "hi:n:f:", ["help", "input=", "nprocs=", "nodefile="])
except getopt.GetoptError:
    print ("Usage: ")
    print ('    momap.py -i momap.inp -n 4')
    print ('    momap.py -i momap.inp -f nodefile')
    sys.exit(1)
for opt, arg in opts:
    if opt in ("-h", "--help"):
        print ("Usage: ")
        print ('    momap.py -i momap.inp -n 4')
        print ('    momap.py -i momap.inp -f nodefile')
        sys.exit(0)
    elif opt in ("-i", "--input"):
        inputfile = arg
    elif opt in ("-n", "--nprocs"):
        nprocs = int(arg)
    elif opt in ("-f", "--nodefile"):
        nodefile = arg

if len(nodefile):
    machinefile = nodefile
else:
    if nprocs:
        with open("nodefile", "w") as f:
            for i in range(nprocs):
                f.write("localhost\n")
        machinefile = "nodefile"
    else:
        found = False
        if os.path.isfile(inputfile):
            with open(inputfile) as f:
                if '&transport' in f.read().lower():
                    found = True
        else:
            print ("ERROR: %s not found!" % inputfile)
            sys.exit(1)
        if not found:
            print ("Usage: ")
            print ('    momap.py -i momap.inp -n 4')
            print ('    momap.py -i momap.inp -f nodefile')
            sys.exit(1)

if len(nodefile):
    if nprocs == 0:
        nl = 0
        with open(machinefile, 'r') as f:
            for line in f:
                nl += 1
        nprocs = nl

# -----------------------------------------------------------------------
# Setting local environment
# Local settings are written in the .environment file
#
MOMAP_ROOT = os.getenv("MOMAP_ROOT")
if not MOMAP_ROOT:
    print ('Please set environment variable MOMAP_ROOT!')
    sys.exit(1)

# Set up environments

MOMAP_LICENSE = os.getenv("MOMAP_LICENSE")
if not MOMAP_LICENSE:
    print ('Please obtain a valid license before you can run!')
    sys.exit(1)

MOMAP_LICENSE_NODE = os.getenv("MOMAP_LICENSE_NODE")

# -----------------------------------------------------------------------
# Set the environment variables. No need to change anything below
#
MOMAP_BIN = MOMAP_ROOT + "/bin"
MOMAP_LIB = MOMAP_ROOT + "/lib"

if ( MOMAP_OS == "WIN" ):
    MOMAP_MPI_ROOT = MOMAP_BIN + "/msmpi"
else:
    MOMAP_MPI_ROOT = MOMAP_BIN + "/mpich2"

MOMAP_MPI_BIN = MOMAP_MPI_ROOT + "/bin"
MOMAP_MPI_LIB = MOMAP_MPI_ROOT + "/lib"

os.environ['PATH'] = MOMAP_MPI_BIN + ':' + MOMAP_BIN + ':' + os.environ['PATH']

if 'LD_LIBRARY_PATH' not in os.environ:
    os.environ['LD_LIBRARY_PATH'] = MOMAP_LIB
else:
    os.environ['LD_LIBRARY_PATH'] += ':' + MOMAP_LIB

os.environ['LD_LIBRARY_PATH'] += ':' + MOMAP_MPI_LIB

os.environ['MPICH_PROCESS_GROUP'] = 'yes'

# print os.environ['PATH']
# print os.environ['LD_LIBRARY_PATH']

# initialize all to zero

do_evc = 0
do_spec_tvcf_ft = 0
do_spec_tvcf_spec = 0
do_ic_tvcf_ft = 0
do_ic_tvcf_spec = 0
do_isc_tvcf_ft = 0
do_isc_tvcf_spec = 0
do_spec_sums = 0

# =========================================================

do_transport_flag = 0;

do_transport_prepare = 0
do_transport_submit_HL_job = 0
do_transport_submit_RE_job = 0
do_transport_get_transferintegral = 0
do_transport_get_re_evc = 0
do_transport_run_MC = 0
do_transport_run_MC_temp = 0
do_transport_get_mob_MC = 0
do_transport_get_mob_MC_temp = 0
do_transport_run_ME = 0
do_transport_run_ME_temp = 0
do_transport_get_mob_ME = 0
do_transport_get_mob_ME_temp = 0
do_transport_gather_momap_data = 0

temp_start = 0
temp_end = 0
temp_delta = 0

lat_site_energy = 0

ratetype = 'marcus'
chargetype = 'a'

with open(inputfile, "r") as f:
    lines = f.readlines()
    for line in lines:
        line = line.strip()

        if line.startswith("#"):
            continue

        # get rid of comment starting with !
        linetmp = line.split("!")

        # token before "="
        items = linetmp[0].split("=")

        token = items[0].lower().strip()

        # -----------------------------------------------------------------------
        if token == "do_evc".lower():
            val = items[1].strip()
            do_evc = int(val)
            continue

        if token == "do_spec_tvcf_ft".lower():
            val = items[1].strip()
            do_spec_tvcf_ft = int(val)
            continue

        if token == "do_spec_tvcf_spec".lower():
            val = items[1].strip()
            do_spec_tvcf_spec = int(val)
            continue

        if token == "do_ic_tvcf_ft".lower():
            val = items[1].strip()
            do_ic_tvcf_ft = int(val)
            continue

        if token == "do_ic_tvcf_spec".lower():
            val = items[1].strip()
            do_ic_tvcf_spec = int(val)
            continue

        if token == "do_isc_tvcf_ft".lower():
            val = items[1].strip()
            do_isc_tvcf_ft = int(val)
            continue

        if token == "do_isc_tvcf_spec".lower():
            val = items[1].strip()
            do_isc_tvcf_spec = int(val)
            continue

        if token == "do_spec_sums".lower():
            val = items[1].strip()
            do_spec_sums = int(val)
            continue

        # -----------------------------------------------------------------------
        if token == "reduce_eps".lower():
            val = items[1].strip()
            reduce_eps = val
            continue

        if token == "if_cal_ic".lower():
            val = items[1].strip()
            if_cal_ic = val
            continue

        # =======================================================================

        if token == "&transport".lower():
            do_transport_flag = 1
            do_transport_gather_momap_data = 1
            continue

        if token == "do_transport_prepare".lower():
            val = items[1].strip()
            do_transport_prepare = int(val)
            continue

        if token == "do_transport_submit_HL_job".lower():
            val = items[1].strip()
            do_transport_submit_HL_job = int(val)
            continue

        if token == "do_transport_submit_RE_job".lower():
            val = items[1].strip()
            do_transport_submit_RE_job = int(val)
            continue

        if token == "do_transport_get_transferintegral".lower():
            val = items[1].strip()
            do_transport_get_transferintegral = int(val)
            continue

        if token == "do_transport_get_re_evc".lower():
            val = items[1].strip()
            do_transport_get_re_evc = int(val)
            continue

        if token == "do_transport_run_MC_temp".lower():
            val = items[1].strip()
            do_transport_run_MC_temp = int(val)
            continue

        if token == "do_transport_get_mob_MC_temp".lower():
            val = items[1].strip()
            do_transport_get_mob_MC_temp = int(val)
            continue

        if token == "do_transport_run_MC".lower():
            val = items[1].strip()
            do_transport_run_MC = int(val)
            continue

        if token == "do_transport_get_mob_MC".lower():
            val = items[1].strip()
            do_transport_get_mob_MC = int(val)
            continue

        if token == "do_transport_run_ME_temp".lower():
            val = items[1].strip()
            do_transport_run_ME_temp = int(val)
            continue

        if token == "do_transport_get_mob_ME_temp".lower():
            val = items[1].strip()
            do_transport_get_mob_ME_temp = int(val)
            continue

        if token == "do_transport_run_ME".lower():
            val = items[1].strip()
            do_transport_run_ME = int(val)
            continue

        if token == "do_transport_get_mob_ME".lower():
            val = items[1].strip()
            do_transport_get_mob_ME = int(val)
            continue

        if token == "do_transport_gather_momap_data".lower():
            val = items[1].strip()
            do_transport_gather_momap_data = int(val)
            continue

        if token == "lat_site_energy".lower():
            val = items[1].strip()
            lat_site_energy = int(val)
            continue

        if token == "start_temp".lower():
            val = items[1].strip()
            temp_start = float(val)
            continue

        if token == "end_temp".lower():
            val = items[1].strip()
            temp_end = float(val)
            continue

        if token == "delta_temp".lower():
            val = items[1].strip()
            temp_delta = float(val)
            continue

        if token == "ratetype".lower():
            val = items[1].replace('"','').replace("'","").strip()
            ratetype = val
            continue

        if token == "chargetype".lower():
            val = items[1].replace('"','').replace("'","").strip()
            chargetype = val.lower()
            continue


if do_transport_flag == 0:
    if len(machinefile) == 0:
        print ("Usage: ")
        print ('    momap -i momap.inp -n 4')
        print ('    momap -i momap.inp -f nodefile')
        sys.exit(0)


# -----------------------------------------------------------------------
# Print title
sys.stdout.write("  __  __    ___    __  __      _      ____  \n")
sys.stdout.write(" |  \/  |  / _ \  |  \/  |    / \    |  _ \ \n")
sys.stdout.write(" | |\/| | | | | | | |\/| |   / _ \   | |_) |\n")
sys.stdout.write(" | |  | | | |_| | | |  | |  / ___ \  |  __/ \n")
sys.stdout.write(" |_|  |_|  \___/  |_|  |_| /_/   \_\ |_|       Version 2019B (2.1.2)\n")
sys.stdout.write("\n")
sys.stdout.write(" Copyright (c) 2017 Shuaigroup @ Tsinghua University &\n")
sys.stdout.write("         Institute of Chemistry, Chinese Academy of Sciences.\n")
sys.stdout.write("         All Rights Reserved.\n")
sys.stdout.write("\n")
sys.stdout.flush()

# -----------------------------------------------------------------------
# Vibrational analysis
if do_evc:
    print ("Do evc calculation")
    com = MOMAP_BIN + '/evc.exe ' + inputfile
    print (com)
    sys.stdout.flush()
    ret = os.system(com)
    if ret:
        print ("Terminated with ERROR for evc calculation")
        print ("STOP")
        sys.exit(1)
    else:
        print ("Normal finish of evc calculation")
    sys.stdout.flush()

if do_spec_tvcf_ft:
    print ("Do spec_ft calculation")
    if MOMAP_OS == "WIN":
        # com = MOMAP_BIN + '/TVCORF_SPEC.exe ' + inputfile
        com = MOMAP_MPI_BIN + '/mpiexec -n ' + str(nprocs) + ' '
        com += MOMAP_BIN + '/TVCORF_SPEC_para.exe ' + inputfile
    else:
        com = MOMAP_MPI_BIN + '/mpiexec -machinefile ' + machinefile + ' '
        com += MOMAP_BIN + '/TVCORF_SPEC_para.exe ' + inputfile
    print (com)
    sys.stdout.flush()
    ret = os.system(com)
    if ret:
        print ("Terminated with ERROR for spec_ft calculation")
        print ("STOP")
        sys.exit(1)
    else:
        print ("Normal finish of spec_ft calculation")
    sys.stdout.flush()

if do_spec_tvcf_spec:
    print ("Do spec_spec calculation")
    com = MOMAP_BIN + '/TVSPEC_SPEC.exe ' + inputfile
    print (com)
    sys.stdout.flush()
    ret = os.system(com)
    if ret:
        print ("Terminated with ERROR for spec_spec calculation")
        print ("STOP")
        sys.exit(1)
    else:
        print ("Normal finish of spec_spec calculation")
    sys.stdout.flush()

# # Correlation calculation by TVCF
if do_ic_tvcf_ft:
    print ("Do ic_ft calculation")
    if MOMAP_OS == "WIN":
        # com = MOMAP_BIN + '/TVCORF_IC.exe ' + inputfile
        com = MOMAP_MPI_BIN + '/mpiexec -n ' + str(nprocs) + ' '
        com += MOMAP_BIN + '/TVCORF_IC_para.exe ' + inputfile
    else:
        com = MOMAP_MPI_BIN + '/mpiexec -machinefile ' + machinefile + ' '
        com += MOMAP_BIN + '/TVCORF_IC_para.exe ' + inputfile
    print (com)
    sys.stdout.flush()
    ret = os.system(com)
    if ret:
        print ("Terminated with ERROR for ic_ft calculation")
        print ("STOP")
        sys.exit(1)
    else:
        print ("Normal finish of ic_ft calculation")
    sys.stdout.flush()

if do_ic_tvcf_spec:
    print ("Do ic_spec calculation")
    com = MOMAP_BIN + '/TVSPEC_IC.exe ' + inputfile
    print (com)
    sys.stdout.flush()
    ret = os.system(com)
    if ret:
        print ("Terminated with ERROR for ic_spec calculation")
        print ("STOP")
        sys.exit(1)
    else:
        print ("Normal finish of ic_spec calculation")
    sys.stdout.flush()

if do_isc_tvcf_ft:
    print ("Do isc_ft calculation")
    if MOMAP_OS == "WIN":
        # com = MOMAP_BIN + '/TVCORF_ISC.exe ' + inputfile
        com = MOMAP_MPI_BIN + '/mpiexec -n ' + str(nprocs) + ' '
        com += MOMAP_BIN + '/TVCORF_ISC_para.exe ' + inputfile
    else:
        com = MOMAP_MPI_BIN + '/mpiexec -machinefile ' + machinefile + ' '
        com += MOMAP_BIN + '/TVCORF_ISC_para.exe ' + inputfile
    print (com)
    sys.stdout.flush()
    ret = os.system(com)
    if ret:
        print ("Terminated with ERROR for isc_ft calculation")
        print ("STOP")
        sys.exit(1)
    else:
        print ("Normal finish of isc_ft calculation")
    sys.stdout.flush()


if do_isc_tvcf_spec:
    print ("Do isc_spec calculation")
    com = MOMAP_BIN + '/TVSPEC_ISC.exe ' + inputfile
    print (com)
    sys.stdout.flush()
    ret = os.system(com)
    if ret:
        print ("Terminated with ERROR for isc_spec calculation")
        print ("STOP")
        sys.exit(1)
    else:
        print ("Normal finish of isc_spec calculation")
    sys.stdout.flush()

if do_spec_sums:
    print ("Do spec_sums calculation")
    if MOMAP_OS == "WIN":
        com = MOMAP_MPI_BIN + '/mpiexec -n ' + str(nprocs) + ' '
        com += MOMAP_BIN + '/sumstat_para.exe ' + inputfile
    else:
        com = MOMAP_MPI_BIN + '/mpiexec -machinefile ' + machinefile + ' '
        com += MOMAP_BIN + '/sumstat_para.exe ' + inputfile
    print (com)
    sys.stdout.flush()
    os.system(com)
    # --------------------------------------------------------------------
    print ("> reduce_num spec.sums.abs.dat %s" % reduce_eps)
    com = MOMAP_BIN + '/reduce_num.exe spec.sums.abs.dat ' + reduce_eps
    sys.stdout.flush()
    print (com)
    ret = os.system(com)
    # --------------------------------------------------------------------
    print ("> reduce_num spec.sums.emi.dat %s" % reduce_eps)
    com = MOMAP_BIN + '/reduce_num.exe spec.sums.emi.dat ' + reduce_eps
    sys.stdout.flush()
    print (com)
    ret = os.system(com)
    # --------------------------------------------------------------------
    if if_cal_ic == ".t." or if_cal_ic == ".T." or \
        if_cal_ic == "t" or if_cal_ic == "T" or \
        if_cal_ic == ".true." or if_cal_ic == ".TRUE.":
        print ("> reduce_num spec.sums.ic.dat %s" % reduce_eps)
        com = MOMAP_BIN + '/reduce_num.exe spec.sums.ic.dat ' + reduce_eps
        print (com)
        sys.stdout.flush()
        ret = os.system(com)
    if ret:
        print ("Terminated with ERROR for spec_sums calculation")
        print ("STOP")
        sys.exit(1)
    else:
        print ("Normal finish of spec_sums calculation")
    sys.stdout.flush()

# ========================================================================
# Module transport

def wait_running():
    """Wait for running processes to end."""
    RUNDIR = WORKDIR + "/RUN"

    nrun = len(glob.glob1(RUNDIR, "running.*"))
    while nrun > 0:
        time.sleep(1)
        nrun = len(glob.glob1(RUNDIR, "running.*"))

        nerror = len(glob.glob1(RUNDIR, "ERROR-*"))
        if nerror > 0:
            sys.stderr.write("\nERROR occurs in running Momap calculations.\n")
            sys.stderr.write("Check RUN/ERROR-* to locate the problem.\n")
            sys.stderr.write("STOP\n\n")
            sys.exit(1);


def usescr(script):
    """Run script."""
    cmd = "./scr/" + script
    os.system(cmd)


def clean_running_flags():
    running_files = WORKDIR + "/RUN/*"
    files = glob.glob(running_files);
    if len(files) > 0:
        cmd = "rm " + running_files;
        os.system(cmd);


clean_running_flags()


if do_transport_prepare:
    sys.stdout.write("-----------------------------------------------------\n")
    sys.stdout.write("Do transport preparation...\n")

    rundir = WORKDIR + '/.'
    sys.stdout.write("\n>> Run at directory: %s\n" % (rundir))

    cmd = MOMAP_BIN + '/transport_prepare.exe'
    sys.stdout.write("\n$> transport_prepare.exe\n")
    sys.stdout.flush()
    ret = os.system(cmd)
    if ret:
        sys.stderr.write("Terminated with ERROR for %s.\n" % cmd)
        sys.stderr.write("Current directory: %s\n" % os.getcwd())
        sys.stderr.write("STOP\n")
        sys.exit(1)
    else:
        sys.stdout.write("\n--- Normal end of transport preparation.\n\n")
    sys.stdout.flush()

if do_transport_submit_HL_job:
    sys.stdout.write("-----------------------------------------------------\n")
    sys.stdout.write("Submit jobs for transfer integral calculations...\n")

    rundir = WORKDIR + '/.'
    sys.stdout.write("\n>> Run at directory: %s\n" % (rundir))

    cmd = "python scr/mol_one_uc.py"
    sys.stdout.write("\n$> %s\n\n" % cmd)
    sys.stdout.flush()
    ret = os.system(cmd)
    if ret:
        sys.stderr.write("Terminated with ERROR for %s.\n" % cmd)
        sys.stderr.write("Current directory: %s\n" % os.getcwd())
        sys.stderr.write("STOP\n")
        sys.exit(1)

    # wait_running()

    cmd = "python scr/mol_one_nei.py"
    sys.stdout.write("\n$> %s\n\n" % cmd)
    sys.stdout.flush()
    ret = os.system(cmd)
    if ret:
        sys.stderr.write("Terminated with ERROR for %s.\n" % cmd)
        sys.stderr.write("Current directory: %s\n" % os.getcwd())
        sys.stderr.write("STOP\n")
        sys.exit(1)

    # wait_running()

    cmd = "python scr/mol_two.py"
    sys.stdout.write("\n$> %s\n\n" % cmd)
    sys.stdout.flush()
    ret = os.system(cmd)
    if ret:
        sys.stderr.write("Terminated with ERROR for %s.\n" % cmd)
        sys.stderr.write("Current directory: %s\n" % os.getcwd())
        sys.stderr.write("STOP\n")
        sys.exit(1)

    wait_running()

    sys.stdout.write("\n--- Normal end of transfer integral calculations.\n\n")
    sys.stdout.flush()

if do_transport_get_transferintegral:
    sys.stdout.write("-----------------------------------------------------\n")
    sys.stdout.write("Collect transfer integrals...\n")

    rundir = WORKDIR + '/.'
    sys.stdout.write("\n>> Run at directory: %s\n" % (rundir))

    cmd = "python scr/get_transint.py"
    sys.stdout.write("\n$> %s\n\n" % cmd)
    sys.stdout.flush()
    ret = os.system(cmd)
    if ret:
        sys.stderr.write("Terminated with ERROR for %s.\n" % cmd)
        sys.stderr.write("Current directory: %s\n" % os.getcwd())
        sys.stderr.write("STOP\n")
        sys.exit(1)

    wait_running()

    sys.stdout.write("\n--- Normal end of collecting transfer integrals.\n\n")
    sys.stdout.flush()

if do_transport_submit_RE_job:
    sys.stdout.write("-----------------------------------------------------\n")
    sys.stdout.write("Submit jobs for reorganization energy calculation...\n")

    rundir = WORKDIR + '/.'
    sys.stdout.write("\n>> Run at directory: %s\n" % (rundir))

    cmd = "python scr/mol_RE.py"
    sys.stdout.write("\n$> %s\n\n" % cmd)
    sys.stdout.flush()
    ret = os.system(cmd)
    if ret:
        sys.stderr.write("Terminated with ERROR for %s.\n" % cmd)
        sys.stderr.write("Current directory: %s\n" % os.getcwd())
        sys.stderr.write("STOP\n")
        sys.exit(1)

    wait_running()

    sys.stdout.write("\n--- Normal end of reorganization energy calculations.\n\n")
    sys.stdout.flush()

if lat_site_energy:
    sys.stdout.write("-----------------------------------------------------\n")
    sys.stdout.write("Collect site energies...\n")

    rundir = WORKDIR + '/.'
    sys.stdout.write("\n>> Run at directory: %s\n" % (rundir))

    cmd = "python scr/get_Einternal.py"
    sys.stdout.write("\n$> %s\n\n" % cmd)
    sys.stdout.flush()
    ret = os.system(cmd)
    if ret:
        sys.stderr.write("Terminated with ERROR for %s.\n" % cmd)
        sys.stderr.write("Current directory: %s\n" % os.getcwd())
        sys.stderr.write("STOP\n")
        sys.exit(1)

    wait_running()

    sys.stdout.write("\n--- Normal end of collecting site energies.\n\n")
    sys.stdout.flush()

if do_transport_get_re_evc:
    sys.stdout.write("-----------------------------------------------------\n")
    sys.stdout.write("Analyze reorganization energies...\n")

    rundir = WORKDIR + '/.'
    sys.stdout.write("\n>> Run at directory: %s\n" % (rundir))

    cmd = "python scr/prepare_RE.py"
    sys.stdout.write("\n$> %s\n\n" % cmd)
    sys.stdout.flush()
    ret = os.system(cmd)
    if ret:
        sys.stderr.write("Terminated with ERROR for %s.\n" % cmd)
        sys.stderr.write("Current directory: %s\n" % os.getcwd())
        sys.stderr.write("STOP\n")
        sys.exit(1)

    cmd = "python scr/run_RE.py"
    sys.stdout.write("\n$> %s\n\n" % cmd)
    sys.stdout.flush()
    ret = os.system(cmd)
    if ret:
        sys.stderr.write("Terminated with ERROR for %s.\n" % cmd)
        sys.stderr.write("Current directory: %s\n" % os.getcwd())
        sys.stderr.write("STOP\n")
        sys.exit(1)

    wait_running()

    cmd = "python scr/get_RE.py"
    sys.stdout.write("\n$> %s\n\n" % cmd)
    sys.stdout.flush()
    ret = os.system(cmd)
    if ret:
        sys.stderr.write("Terminated with ERROR for %s.\n" % cmd)
        sys.stderr.write("Current directory: %s\n" % os.getcwd())
        sys.stderr.write("STOP\n")
        sys.exit(1)

    sys.stdout.write("\n--- Normal end of analyzing reorganization energies.\n\n")
    sys.stdout.flush()

if do_transport_run_MC:
    sys.stdout.write("-----------------------------------------------------\n")
    sys.stdout.write("Run Monte Carlo (MC) simulations...\n")

    rootdir = 'MC-' + ratetype
    rundir = WORKDIR + '/' + rootdir

    sys.stdout.write("Run at directory: %s\n" % (rundir))

    if os.path.exists(rundir) == False:
        path_save = os.getcwd()
        os.chdir(WORKDIR)
        cmd = MOMAP_BIN + '/transport_prepare.exe -info false'
        sys.stdout.write("\n$> transport_prepare.exe\n")
        sys.stdout.flush()
        ret = os.system(cmd)
        if ret:
            sys.stderr.write("Terminated with ERROR for %s.\n" % cmd)
            sys.stderr.write("Current directory: %s\n" % os.getcwd())
            sys.stderr.write("STOP\n")
            sys.exit(1)
        sys.stdout.flush()
        os.chdir(path_save)

    if 'e' in chargetype or 'a' in chargetype:
        shortrundir = rootdir + '/elec'
        sys.stdout.write("\n>> Run at directory: %s\n" % shortrundir)

        rundir = WORKDIR + '/' + shortrundir

        os.chdir(rundir)
        cmd = "python prepare-mc.py"
        sys.stdout.write("\n$> %s\n\n" % cmd)
        sys.stdout.flush()
        ret = os.system(cmd)
        if ret:
            sys.stderr.write("Terminated with ERROR for %s.\n" % cmd)
            sys.stderr.write("Current directory: %s\n" % os.getcwd())
            sys.stderr.write("STOP\n")
            sys.exit(1)

        os.chdir(rundir)
        cmd = "python run_mc_batch.py"
        sys.stdout.write("\n$> %s\n\n" % cmd)
        sys.stdout.flush()
        ret = os.system(cmd)
        if ret:
            sys.stderr.write("Terminated with ERROR for %s.\n" % cmd)
            sys.stderr.write("Current directory: %s\n" % os.getcwd())
            sys.stderr.write("STOP\n")
            sys.exit(1)

    if 'h' in chargetype or 'a' in chargetype:
        shortrundir = rootdir + '/hole'
        sys.stdout.write("\n>> Run at directory: %s\n" % shortrundir)

        rundir = WORKDIR + '/' + shortrundir

        os.chdir(rundir)
        cmd = "python prepare-mc.py"
        sys.stdout.write("\n$> %s\n\n" % cmd)
        sys.stdout.flush()
        ret = os.system(cmd)
        if ret:
            sys.stderr.write("Terminated with ERROR for %s.\n" % cmd)
            sys.stderr.write("Current directory: %s\n" % os.getcwd())
            sys.stderr.write("STOP\n")
            sys.exit(1)

        os.chdir(rundir)
        cmd = "python run_mc_batch.py"
        sys.stdout.write("\n$> %s\n\n" % cmd)
        sys.stdout.flush()
        ret = os.system(cmd)
        if ret:
            sys.stderr.write("Terminated with ERROR for %s.\n" % cmd)
            sys.stderr.write("Current directory: %s\n" % os.getcwd())
            sys.stderr.write("STOP\n")
            sys.exit(1)

    wait_running()

    sys.stdout.write("\n--- Normal end of Monte Carlo (MC) simulations.\n\n")
    sys.stdout.flush()

if do_transport_get_mob_MC:
    sys.stdout.write("-----------------------------------------------------\n")
    sys.stdout.write("Calculate Random Walk mobilities...\n")

    rootdir = 'MC-' + ratetype
    rundir = WORKDIR + '/' + rootdir

    sys.stdout.write("Run at directory: %s\n" % (rundir))

    if 'e' in chargetype or 'a' in chargetype:
        shortrundir = rootdir + '/elec'
        sys.stdout.write("\n>> Run at directory: %s\n" % shortrundir)

        rundir = WORKDIR + '/' + shortrundir

        os.chdir(rundir)
        cmd = "python get_mob.py"
        sys.stdout.write("\n$> %s\n\n" % cmd)
        sys.stdout.flush()
        ret = os.system(cmd)
        if ret:
            sys.stderr.write("Terminated with ERROR for %s.\n" % cmd)
            sys.stderr.write("Current directory: %s\n" % os.getcwd())
            sys.stderr.write("STOP\n")
            sys.exit(1)

    if 'h' in chargetype or 'a' in chargetype:
        shortrundir = rootdir + '/hole'
        sys.stdout.write("\n>> Run at directory: %s\n" % shortrundir)

        rundir = WORKDIR + '/' + shortrundir

        os.chdir(rundir)
        cmd = "python get_mob.py"
        sys.stdout.write("\n$> %s\n\n" % cmd)
        sys.stdout.flush()
        ret = os.system(cmd)
        if ret:
            sys.stderr.write("Terminated with ERROR for %s.\n" % cmd)
            sys.stderr.write("Current directory: %s\n" % os.getcwd())
            sys.stderr.write("STOP\n")
            sys.exit(1)

    wait_running()

    sys.stdout.write("\n--- Normal end of Random Walk mobilities.\n\n")
    sys.stdout.flush()

if do_transport_run_MC_temp:
    sys.stdout.write("-----------------------------------------------------\n")
    sys.stdout.write("Run Monte Carlo with TD simulations...\n")

    rootdir = 'MC-' + ratetype + '-temp'
    rundir = WORKDIR + '/' + rootdir

    sys.stdout.write("Run at directory: %s\n" % (rundir))

    if os.path.exists(rundir) == False:
        path_save = os.getcwd()
        os.chdir(WORKDIR)
        cmd = MOMAP_BIN + '/transport_prepare.exe -info false'
        sys.stdout.write("\n$> transport_prepare.exe\n")
        sys.stdout.flush()
        ret = os.system(cmd)
        if ret:
            sys.stderr.write("Terminated with ERROR for %s.\n" % cmd)
            sys.stderr.write("Current directory: %s\n" % os.getcwd())
            sys.stderr.write("STOP\n")
            sys.exit(1)
        sys.stdout.flush()
        os.chdir(path_save)

    if temp_start and temp_delta:
        ntemp = int((temp_end - temp_start) / temp_delta + 1)
        for i in range(ntemp):
            temp = str(int(temp_start + i * temp_delta))

            if 'e' in chargetype or 'a' in chargetype:
                shortrundir = rootdir + '/' + temp + '/elec'
                sys.stdout.write("\n>> Run at directory: %s\n" % (shortrundir))

                rundir = WORKDIR + '/' + shortrundir

                os.chdir(rundir)
                cmd = "python prepare-mc.py"
                sys.stdout.write("\n$> %s\n\n" % cmd)
                sys.stdout.flush()
                ret = os.system(cmd)
                if ret:
                    sys.stderr.write("Terminated with ERROR for prepare-mc.py (temp).\n")
                    sys.stderr.write("Current directory: %s\n" % os.getcwd())
                    sys.stderr.write("STOP\n")
                    sys.exit(1)

                os.chdir(rundir)
                cmd = "python run_mc_batch.py"
                sys.stdout.write("\n$> %s\n\n" % cmd)
                sys.stdout.flush()
                ret = os.system(cmd)
                if ret:
                    sys.stderr.write("Terminated with ERROR for run_mc_batch.py (temp).\n")
                    sys.stderr.write("Current directory: %s\n" % os.getcwd())
                    sys.stderr.write("STOP\n")
                    sys.exit(1)

            if 'h' in chargetype or 'a' in chargetype:
                shortrundir = rootdir + '/' + temp + '/hole'
                sys.stdout.write("\n>> Run at directory: %s\n" % (shortrundir))

                rundir = WORKDIR + '/' + shortrundir

                os.chdir(rundir)
                cmd = "python prepare-mc.py"
                sys.stdout.write("\n$> %s\n\n" % cmd)
                sys.stdout.flush()
                ret = os.system(cmd)
                if ret:
                    sys.stderr.write("Terminated with ERROR for prepare-mc.py (temp).\n")
                    sys.stderr.write("Current directory: %s\n" % os.getcwd())
                    sys.stderr.write("STOP\n")
                    sys.exit(1)

                os.chdir(rundir)
                cmd = "python run_mc_batch.py"
                sys.stdout.write("\n$> %s\n\n" % cmd)
                sys.stdout.flush()
                ret = os.system(cmd)
                if ret:
                    sys.stderr.write("Terminated with ERROR for run_mc_batch.py (temp).\n")
                    sys.stderr.write("Current directory: %s\n" % os.getcwd())
                    sys.stderr.write("STOP\n")
                    sys.exit(1)

    wait_running()

    sys.stdout.write("\n--- Normal end of Monte Carlo with TD simulations.\n\n")
    sys.stdout.flush()

if do_transport_get_mob_MC_temp:
    sys.stdout.write("-----------------------------------------------------\n")
    sys.stdout.write("Calculate Random Walk with TD mobilities...\n")

    ehtypes = ['elec', 'hole']
    rootdir = 'MC-' + ratetype + '-temp'
    rundir = WORKDIR + '/' + rootdir

    sys.stdout.write("Run at directory: %s\n" % (rundir))

    if temp_start and temp_delta:
        ntemp = int((temp_end - temp_start) / temp_delta + 1)
        for i in range(ntemp):
            temp = str(int(temp_start + i * temp_delta))

            if 'e' in chargetype or 'a' in chargetype:
                shortrundir = rootdir + '/' + temp + '/elec'
                sys.stdout.write("\n>> Run at directory: %s\n" % (shortrundir))

                rundir = WORKDIR + '/' + shortrundir

                os.chdir(rundir)
                cmd = "python get_mob.py"
                sys.stdout.write("\n$> %s\n" % cmd)
                sys.stdout.flush()
                ret = os.system(cmd)
                if ret:
                    sys.stderr.write("Terminated with ERROR for get_mob.py (temp).\n")
                    sys.stderr.write("Current directory: %s\n" % os.getcwd())
                    sys.stderr.write("STOP\n")
                    sys.exit(1)

            if 'h' in chargetype or 'a' in chargetype:
                shortrundir = rootdir + '/' + temp + '/hole'
                sys.stdout.write("\n>> Run at directory: %s\n" % (shortrundir))

                rundir = WORKDIR + '/' + shortrundir

                os.chdir(rundir)
                cmd = "python get_mob.py"
                sys.stdout.write("\n$> %s\n" % cmd)
                sys.stdout.flush()
                ret = os.system(cmd)
                if ret:
                    sys.stderr.write("Terminated with ERROR for get_mob.py (temp).\n")
                    sys.stderr.write("Current directory: %s\n" % os.getcwd())
                    sys.stderr.write("STOP\n")
                    sys.exit(1)

    wait_running()

    sys.stdout.write("\n--- Normal end of Random Walk mobilities with TD.\n\n")
    sys.stdout.flush()

if do_transport_run_ME:
    sys.stdout.write("-----------------------------------------------------\n")
    sys.stdout.write("Run Master Equation (ME) simulation...\n")

    rootdir = 'ME-' + ratetype
    rundir = WORKDIR + '/' + rootdir

    sys.stdout.write("Run at directory: %s\n" % (rundir))

    if os.path.exists(rundir) == False:
        path_save = os.getcwd()
        os.chdir(WORKDIR)
        cmd = MOMAP_BIN + '/transport_prepare.exe -info false'
        sys.stdout.write("\n$> transport_prepare.exe\n")
        sys.stdout.flush()
        ret = os.system(cmd)
        if ret:
            sys.stderr.write("Terminated with ERROR for %s.\n" % cmd)
            sys.stderr.write("Current directory: %s\n" % os.getcwd())
            sys.stderr.write("STOP\n")
            sys.exit(1)
        sys.stdout.flush()
        os.chdir(path_save)

    if 'e' in chargetype or 'a' in chargetype:
        shortrundir = rootdir + '/elec'
        sys.stdout.write("\n>> Run at directory: %s\n" % (shortrundir))

        rundir = WORKDIR + '/' + shortrundir

        os.chdir(rundir)
        cmd = "python prepare-me.py"
        sys.stdout.write("\n$> %s\n\n" % cmd)
        sys.stdout.flush()
        ret = os.system(cmd)
        if ret:
            sys.stderr.write("Terminated with ERROR for %s.\n" % cmd)
            sys.stderr.write("Current directory: %s\n" % os.getcwd())
            sys.stderr.write("STOP\n")
            sys.exit(1)

        os.chdir(rundir)
        cmd = "python run_me_batch.py"
        sys.stdout.write("\n$> %s\n\n" % cmd)
        sys.stdout.flush()
        ret = os.system(cmd)
        if ret:
            sys.stderr.write("Terminated with ERROR for run_me_batch.py.\n")
            sys.stderr.write("Current directory: %s\n" % os.getcwd())
            sys.stderr.write("STOP\n")
            sys.exit(1)

    if 'h' in chargetype or 'a' in chargetype:
        shortrundir = rootdir + '/hole'
        sys.stdout.write("\n>> Run at directory: %s\n" % (shortrundir))

        rundir = WORKDIR + '/' + shortrundir

        os.chdir(rundir)
        cmd = "python prepare-me.py"
        sys.stdout.write("\n$> %s\n\n" % cmd)
        sys.stdout.flush()
        ret = os.system(cmd)
        if ret:
            sys.stderr.write("Terminated with ERROR for %s.\n" % cmd)
            sys.stderr.write("Current directory: %s\n" % os.getcwd())
            sys.stderr.write("STOP\n")
            sys.exit(1)

        os.chdir(rundir)
        cmd = "python run_me_batch.py"
        sys.stdout.write("\n$> %s\n\n" % cmd)
        sys.stdout.flush()
        ret = os.system(cmd)
        if ret:
            sys.stderr.write("Terminated with ERROR for run_me_batch.py.\n")
            sys.stderr.write("Current directory: %s\n" % os.getcwd())
            sys.stderr.write("STOP\n")
            sys.exit(1)

    wait_running()

    sys.stdout.write("\n--- Normal end of Master Equation (ME) simulations.\n\n")
    sys.stdout.flush()

if do_transport_get_mob_ME:
    sys.stdout.write("-----------------------------------------------------\n")
    sys.stdout.write("Calculate drift mobilities from ME results...\n")

    shortrundir = 'ME-' + ratetype
    rundir = WORKDIR + '/' + shortrundir

    sys.stdout.write("Run at directory: %s\n" % (rundir))

    os.chdir(rundir)
    cmd = "python get_mob.py"
    sys.stdout.write("\n$> %s\n" % cmd)
    sys.stdout.flush()
    ret = os.system(cmd)
    if ret:
        sys.stderr.write("Terminated with ERROR for %s.\n" % cmd)
        sys.stderr.write("Current directory: %s\n" % os.getcwd())
        sys.stderr.write("STOP\n")
        sys.exit(1)

    wait_running()

    sys.stdout.write("\n--- Normal end of drift mobility by ME calculation.\n\n")
    sys.stdout.flush()

if do_transport_run_ME_temp:
    sys.stdout.write("-----------------------------------------------------\n")
    sys.stdout.write("Run Master Equation (ME) with TD simulation...\n")

    rootdir = 'ME-' + ratetype + '-temp'
    rundir = WORKDIR + '/' + rootdir

    sys.stdout.write("Run at directory: %s\n" % (rundir))

    if os.path.exists(rundir) == False:
        path_save = os.getcwd()
        os.chdir(WORKDIR)
        cmd = MOMAP_BIN + '/transport_prepare.exe -info false'
        sys.stdout.write("\n$> transport_prepare.exe\n")
        sys.stdout.flush()
        ret = os.system(cmd)
        if ret:
            sys.stderr.write("Terminated with ERROR for %s.\n" % cmd)
            sys.stderr.write("Current directory: %s\n" % os.getcwd())
            sys.stderr.write("STOP\n")
            sys.exit(1)
        sys.stdout.flush()
        os.chdir(path_save)

    if 'e' in chargetype or 'a' in chargetype:
        ntemp = int((temp_end - temp_start) / temp_delta + 1)
        for t in range(ntemp):
            temp = str(int(temp_start + t * temp_delta))
            shortrundir = rootdir + '/elec/' + temp
            sys.stdout.write("\n>> Run at directory: %s\n" % (shortrundir))

            rundir = WORKDIR + '/' + shortrundir

            os.chdir(rundir)
            cmd = "python prepare-me.py"
            sys.stdout.write("\n$> %s\n\n" % cmd)
            sys.stdout.flush()
            ret = os.system(cmd)
            if ret:
                sys.stderr.write("Terminated with ERROR for prepare-me.py (temp).\n")
                sys.stderr.write("Current directory: %s\n" % os.getcwd())
                sys.stderr.write("STOP\n")
                sys.exit(1)

            os.chdir(rundir)
            cmd = "python run_me_batch.py"
            sys.stdout.write("\n$> %s\n\n" % cmd)
            sys.stdout.flush()
            ret = os.system(cmd)
            if ret:
                sys.stderr.write("Terminated with ERROR for run_me_batch.py (temp).\n")
                sys.stderr.write("STOP\n")
                sys.exit(1)

    if 'h' in chargetype or 'a' in chargetype:
        ntemp = int((temp_end - temp_start) / temp_delta + 1)
        for t in range(ntemp):
            temp = str(int(temp_start + t * temp_delta))
            shortrundir = rootdir + '/hole/' + temp
            sys.stdout.write("\n>> Run at directory: %s\n" % (shortrundir))

            rundir = WORKDIR + '/' + shortrundir

            os.chdir(rundir)
            cmd = "python prepare-me.py"
            sys.stdout.write("\n$> %s\n\n" % cmd)
            sys.stdout.flush()
            ret = os.system(cmd)
            if ret:
                sys.stderr.write("Terminated with ERROR for prepare-me.py (temp).\n")
                sys.stderr.write("Current directory: %s\n" % os.getcwd())
                sys.stderr.write("STOP\n")
                sys.exit(1)

            os.chdir(rundir)
            cmd = "python run_me_batch.py"
            sys.stdout.write("\n$> %s\n\n" % cmd)
            sys.stdout.flush()
            ret = os.system(cmd)
            if ret:
                sys.stderr.write("Terminated with ERROR for run_me_batch.py (temp).\n")
                sys.stderr.write("STOP\n")
                sys.exit(1)

    wait_running()

    sys.stdout.write("\n--- Normal end of Master Equation (ME) with TD simulations.\n\n")
    sys.stdout.flush()

if do_transport_get_mob_ME_temp:
    sys.stdout.write("-----------------------------------------------------\n")
    sys.stdout.write("Calculate drift mobilities from ME-TD results...\n")

    shortrundir = 'ME-' + ratetype + '-temp'
    rundir = WORKDIR + '/' + shortrundir

    sys.stdout.write("Run at directory: %s\n" % (rundir))

    os.chdir(rundir)
    cmd = "python get_mob.py"
    sys.stdout.write("\n$> %s\n\n" % cmd)
    sys.stdout.flush()
    ret = os.system(cmd)
    if ret:
        sys.stderr.write("Terminated with ERROR for get_mob.py (temp).\n")
        sys.stderr.write("Current directory: %s\n" % os.getcwd())
        sys.stderr.write("STOP\n")
        sys.exit(1)

    wait_running()

    sys.stdout.write("\n--- Normal end of drift mobility by ME-TD calculation.\n\n")
    sys.stdout.flush()

if do_transport_gather_momap_data:
    sys.stdout.write("-----------------------------------------------------\n")
    sys.stdout.write("Gather Momap data...\n")

    shortrundir = '.'
    rundir = WORKDIR + '/' + shortrundir

    sys.stdout.write("Run at directory: %s\n" % (rundir))

    os.chdir(rundir)
    cmd = MOMAP_BIN + "/transport_gatherdata.exe"
    sys.stdout.write("\n$> transport_gatherdata.exe\n\n")
    sys.stdout.flush()
    ret = os.system(cmd)
    if ret:
        sys.stderr.write("Terminated with ERROR for %s.\n" % (cmd))
        sys.stderr.write("Current directory: %s\n" % os.getcwd())
        sys.stderr.write("STOP\n")
        sys.exit(1)

    sys.stdout.write("\n--- Normal end of Gather Momap data.\n\n")
    sys.stdout.flush()

# -----------------------------------------------------------------------
# Print message
sys.stdout.write("****************************************\n")
sys.stdout.write("*        ALL SUCCESSFULLY DONE.        *\n")
sys.stdout.write("****************************************\n")
sys.stdout.flush()

# ----------------------------------------------------------------------
# Print elapsed time
seconds = time.time() - start_time

seconds_in_day = 86400
seconds_in_hour = 3600
seconds_in_minute = 60

days = seconds // seconds_in_day
seconds = seconds - (days * seconds_in_day)

hours = seconds // seconds_in_hour
seconds = seconds - (hours * seconds_in_hour)

minutes = seconds // seconds_in_minute
seconds = seconds - (minutes * seconds_in_minute)

print("DURATION: {0:.0f} days {1:.0f} hours {2:.0f} minutes {3:.0f} seconds.".format(
    days, hours, minutes, seconds))


